@@ -7,13 +7,30 @@ from time import time
 import datetime
 from dateutil.relativedelta import relativedelta
 from dateutil.tz import tzutc
+import calendar
 
+step_search_re = re.compile(r'^([^-]+)-([^-/]+)(/(.*))?$')
 search_re = re.compile(r'^([^-]+)-([^-/]+)(/(.*))?$')
 only_int_re = re.compile(r'^\d+$')
 any_int_re = re.compile(r'^\d+')
 star_or_int_re = re.compile(r'^(\d+|\*)$')
+VALID_LEN_EXPRESSION = [5, 6]
 
-__all__ = ('croniter',)
+
+class CroniterError(ValueError):
+    pass
+
+
+class CroniterBadCronError(CroniterError):
+    pass
+
+
+class CroniterBadDateError(CroniterError):
+    pass
+
+
+class CroniterNotAlphaError(CroniterError):
+    pass
 
 
 class croniter(object):
@@ -52,91 +69,35 @@ class croniter(object):
     bad_length = 'Exactly 5 or 6 columns has to be specified for iterator' \
                  'expression.'
 
-    def __init__(self, expr_format, start_time=None, ret_type=float):
+    def __init__(self, expr_format, start_time=None, ret_type=float,
+                 day_or=True):
         self._ret_type = ret_type
+        self._day_or = day_or
+
         if start_time is None:
             start_time = time()
 
         self.tzinfo = None
         if isinstance(start_time, datetime.datetime):
             self.tzinfo = start_time.tzinfo
+            # milliseconds/microseconds rounds
+            if start_time.microsecond:
+                start_time = start_time + relativedelta(seconds=1)
             start_time = self._datetime_to_timestamp(start_time)
 
+        self.start_time = start_time
+        self.dst_start_time = start_time
         self.cur = start_time
-        self.exprs = expr_format.split()
 
-        if len(self.exprs) != 5 and len(self.exprs) != 6:
-            raise ValueError(self.bad_length)
+        self.expanded, self.nth_weekday_of_month = self.expand(expr_format)
 
-        expanded = []
-
-        for i, expr in enumerate(self.exprs):
-            e_list = expr.split(',')
-            res = []
-
-            while len(e_list) > 0:
-                e = e_list.pop()
-                t = re.sub(r'^\*(/.+)$', r'%d-%d\1' % (
-                    self.RANGES[i][0],
-                    self.RANGES[i][1]),
-                    str(e))
-                m = search_re.search(t)
-
-                if m:
-                    (low, high, step) = m.group(1), m.group(2), m.group(4) or 1
-
-                    if not any_int_re.search(low):
-                        low = "{0}".format(self.ALPHACONV[i][low.lower()])
-
-                    if not any_int_re.search(high):
-                        high = "{0}".format(self.ALPHACONV[i][high.lower()])
-
-                    if (
-                        not low or not high or int(low) > int(high)
-                        or not only_int_re.search(str(step))
-                    ):
-                        raise ValueError(
-                            "[{0}] is not acceptable".format(expr_format))
-
-                    low, high, step = map(int, [low, high, step])
-                    e_list += range(low, high + 1, step)
-                    # other solution
-                    #try:
-                    #    for j in xrange(int(low), int(high) + 1):
-                    #        if j % int(step) == 0:
-                    #            e_list.append(j)
-                    #except NameError:
-                    #    for j in range(int(low), int(high) + 1):
-                    #        if j % int(step) == 0:
-                    #            e_list.append(j)
-                else:
-                    if not star_or_int_re.search(t):
-                        t = self.ALPHACONV[i][t.lower()]
-
-                    try:
-                        t = int(t)
-                    except:
-                        pass
-
-                    if t in self.LOWMAP[i]:
-                        t = self.LOWMAP[i][t]
-
-                    if (
-                        t not in ["*", "l"]
-                        and (int(t) < self.RANGES[i][0] or
-                             int(t) > self.RANGES[i][1])
-                    ):
-                        raise ValueError(
-                            "[{0}] is not acceptable, out of range".format(
-                                expr_format))
-
-                    res.append(t)
-
-            res.sort()
-            expanded.append(['*'] if (len(res) == 1
-                                      and res[0] == '*')
-                            else res)
-        self.expanded = expanded
+    @classmethod
+    def _alphaconv(cls, index, key, expressions):
+        try:
+            return cls.ALPHACONV[index][key.lower()]
+        except KeyError:
+            raise CroniterNotAlphaError(
+                "[{0}] is not acceptable".format(" ".join(expressions)))
 
     def get_next(self, ret_type=None):
         return self._get_next(ret_type or self._ret_type, is_prev=False)
@@ -146,18 +107,19 @@ class croniter(object):
 
     def get_current(self, ret_type=None):
         ret_type = ret_type or self._ret_type
-        if ret_type == datetime.datetime:
+        if issubclass(ret_type,  datetime.datetime):
             return self._timestamp_to_datetime(self.cur)
         return self.cur
 
-    def _datetime_to_timestamp(self, d):
+    @classmethod
+    def _datetime_to_timestamp(cls, d):
         """
         Converts a `datetime` object `d` into a UNIX timestamp.
         """
         if d.tzinfo is not None:
             d = d.replace(tzinfo=None) - d.utcoffset()
 
-        return self._timedelta_to_seconds(d - datetime.datetime(1970, 1, 1))
+        return cls._timedelta_to_seconds(d - datetime.datetime(1970, 1, 1))
 
     def _timestamp_to_datetime(self, timestamp):
         """
@@ -205,46 +167,74 @@ class croniter(object):
 
     def _get_next(self, ret_type=None, is_prev=False):
         expanded = self.expanded[:]
+        nth_weekday_of_month = self.nth_weekday_of_month.copy()
 
         ret_type = ret_type or self._ret_type
 
-        if ret_type not in (float, datetime.datetime):
+        if not issubclass(ret_type, (float, datetime.datetime)):
             raise TypeError("Invalid ret_type, only 'float' or 'datetime' "
                             "is acceptable.")
 
-        if expanded[2][0] != '*' and expanded[4][0] != '*':
+        # exception to support day of month and day of week as defined in cron
+        if (expanded[2][0] != '*' and expanded[4][0] != '*') and self._day_or:
             bak = expanded[4]
             expanded[4] = ['*']
-            t1 = self._calc(self.cur, expanded, is_prev)
+            t1 = self._calc(self.cur, expanded, nth_weekday_of_month, is_prev)
             expanded[4] = bak
             expanded[2] = ['*']
 
-            t2 = self._calc(self.cur, expanded, is_prev)
+            t2 = self._calc(self.cur, expanded, nth_weekday_of_month, is_prev)
             if not is_prev:
                 result = t1 if t1 < t2 else t2
             else:
                 result = t1 if t1 > t2 else t2
         else:
-            result = self._calc(self.cur, expanded, is_prev)
+            result = self._calc(self.cur, expanded,
+                                nth_weekday_of_month, is_prev)
+
+        # DST Handling for cron job spanning accross days
+        dtstarttime = self._timestamp_to_datetime(self.dst_start_time)
+        dtstarttime_utcoffset = (
+            dtstarttime.utcoffset() or datetime.timedelta(0))
+        dtresult = self._timestamp_to_datetime(result)
+        lag = lag_hours = 0
+        # do we trigger DST on next crontab (handle backward changes)
+        dtresult_utcoffset = dtstarttime_utcoffset
+        if dtresult and self.tzinfo:
+            dtresult_utcoffset = dtresult.utcoffset()
+            lag_hours = (
+                self._timedelta_to_seconds(dtresult - dtstarttime) / (60*60)
+            )
+            lag = self._timedelta_to_seconds(
+                dtresult_utcoffset - dtstarttime_utcoffset
+            )
+        hours_before_midnight = 24 - dtstarttime.hour
+        if dtresult_utcoffset != dtstarttime_utcoffset:
+            if ((lag > 0 and lag_hours >= hours_before_midnight)
+                or (lag < 0 and
+                    ((3600*lag_hours+abs(lag)) >= hours_before_midnight*3600))
+            ):
+                dtresult = dtresult - datetime.timedelta(seconds=lag)
+                result = self._datetime_to_timestamp(dtresult)
+                self.dst_start_time = result
         self.cur = result
-
-        if ret_type == datetime.datetime:
-            result = self._timestamp_to_datetime(result)
-
+        if issubclass(ret_type, datetime.datetime):
+            result = dtresult
         return result
 
-    def _calc(self, now, expanded, is_prev):
+    def _calc(self, now, expanded, nth_weekday_of_month, is_prev):
         if is_prev:
             nearest_diff_method = self._get_prev_nearest_diff
             sign = -1
+            offset = (len(expanded) == 6 or now % 60 > 0) and 1 or 60
         else:
             nearest_diff_method = self._get_next_nearest_diff
             sign = 1
+            offset = (len(expanded) == 6) and 1 or 60
 
-        offset = len(expanded) == 6 and 1 or 60
         dst = now = self._timestamp_to_datetime(now + sign * offset)
 
-        day, month, year = dst.day, dst.month, dst.year
+        month, year = dst.month, dst.year
         current_year = now.year
         DAYS = self.DAYS
 
@@ -275,7 +265,7 @@ class croniter(object):
                 days = DAYS[month - 1]
                 if month == 2 and self.is_leap(year) is True:
                     days += 1
-                if 'l' in expanded[2] and days==d.day:
+                if 'l' in expanded[2] and days == d.day:
                     return False, d
 
                 if is_prev:
@@ -310,6 +300,55 @@ class croniter(object):
                     return True, d
             return False, d
 
+        def proc_day_of_week_nth(d):
+            if '*' in nth_weekday_of_month:
+                s = nth_weekday_of_month['*']
+                for i in range(0, 7):
+                    if i in nth_weekday_of_month:
+                        nth_weekday_of_month[i].update(s)
+                    else:
+                        nth_weekday_of_month[i] = s
+                del nth_weekday_of_month['*']
+
+            candidates = []
+            for wday, nth in nth_weekday_of_month.items():
+                w = (wday + 6) % 7
+                c = calendar.Calendar(w).monthdayscalendar(d.year, d.month)
+                if c[0][0] == 0: c.pop(0)
+                for n in nth:
+                    if len(c) < n:
+                        continue
+                    candidate = c[n - 1][0]
+                    if (
+                        (is_prev and candidate <= d.day) or
+                        (not is_prev and d.day <= candidate)
+                    ):
+                        candidates.append(candidate)
+
+            if not candidates:
+                if is_prev:
+                    d += relativedelta(days=-d.day,
+                                       hour=23, minute=59, second=59)
+                else:
+                    days = DAYS[month - 1]
+                    if month == 2 and self.is_leap(year) is True:
+                        days += 1
+                    d += relativedelta(days=(days - d.day + 1),
+                                       hour=0, minute=0, second=0)
+                return True, d
+
+            candidates.sort()
+            diff_day = (candidates[-1] if is_prev else candidates[0]) - d.day
+            if diff_day != 0:
+                if is_prev:
+                    d += relativedelta(days=diff_day,
+                                       hour=23, minute=59, second=59)
+                else:
+                    d += relativedelta(days=diff_day,
+                                       hour=0, minute=0, second=0)
+                return True, d
+            return False, d
+
         def proc_hour(d):
             if expanded[1][0] != '*':
                 diff_hour = nearest_diff_method(d.hour, expanded[1], 24)
@@ -346,7 +385,8 @@ class croniter(object):
 
         procs = [proc_month,
                  proc_day_of_month,
-                 proc_day_of_week,
+                 (proc_day_of_week_nth if nth_weekday_of_month
+                     else proc_day_of_week),
                  proc_hour,
                  proc_minute,
                  proc_second]
@@ -356,14 +396,16 @@ class croniter(object):
             for proc in procs:
                 (changed, dst) = proc(dst)
                 if changed:
-                    day, month, year = dst.day, dst.month, dst.year
+                    month, year = dst.month, dst.year
                     next = True
                     break
             if next:
                 continue
             return self._datetime_to_timestamp(dst.replace(microsecond=0))
 
-        raise Exception("failed to find prev date")
+        if is_prev:
+            raise CroniterBadDateError("failed to find prev date")
+        raise CroniterBadDateError("failed to find next date")
 
     def _get_next_nearest(self, x, to_check):
         small = [item for item in to_check if item < x]
@@ -399,7 +441,13 @@ class croniter(object):
             return -x
         candidate = candidates[0]
         for c in candidates:
-            if c < range_val:
+            # fixed: c < range_val
+            # this code will reject all 31 day of month, 12 month, 59 second,
+            # 23 hour and so on.
+            # if candidates has just a element, this will not harmful.
+            # but candidates have multiple elements, then values equal to
+            # range_val will rejected.
+            if c <= range_val:
                 candidate = c
                 break
 
@@ -411,9 +459,106 @@ class croniter(object):
         else:
             return False
 
-if __name__ == '__main__':
+    @classmethod
+    def expand(cls, expr_format):
+        expressions = expr_format.split()
 
-    base = datetime.datetime(2010, 1, 25)
-    itr = croniter('0 0 1 * *', base)
-    n1 = itr.get_next(datetime.datetime)
-    print(n1)
+        if len(expressions) not in VALID_LEN_EXPRESSION:
+            raise CroniterBadCronError(cls.bad_length)
+
+        expanded = []
+        nth_weekday_of_month = {}
+
+        for i, expr in enumerate(expressions):
+            e_list = expr.split(',')
+            res = []
+
+            while len(e_list) > 0:
+                e = e_list.pop()
+
+                if i == 4:
+                    e, sep, nth = str(e).partition('#')
+                    if nth and not re.match(r'[1-5]', nth):
+                        raise CroniterBadDateError(
+                            "[{0}] is not acceptable".format(expr_format))
+
+                t = re.sub(r'^\*(\/.+)$', r'%d-%d\1' % (
+                    cls.RANGES[i][0],
+                    cls.RANGES[i][1]),
+                    str(e))
+                m = search_re.search(t)
+
+                if not m:
+                    t = re.sub(r'^(.+)\/(.+)$', r'\1-%d/\2' % (
+                        cls.RANGES[i][1]),
+                        str(e))
+                    m = step_search_re.search(t)
+
+                if m:
+                    (low, high, step) = m.group(1), m.group(2), m.group(4) or 1
+
+                    if not any_int_re.search(low):
+                        low = "{0}".format(cls._alphaconv(i, low, expressions))
+
+                    if not any_int_re.search(high):
+                        high = "{0}".format(cls._alphaconv(i, high, expressions))
+
+                    if (
+                        not low or not high or int(low) > int(high)
+                        or not only_int_re.search(str(step))
+                    ):
+                        raise CroniterBadDateError(
+                            "[{0}] is not acceptable".format(expr_format))
+
+                    low, high, step = map(int, [low, high, step])
+                    rng = range(low, high + 1, step)
+                    e_list += (["{0}#{1}".format(item, nth) for item in rng]
+                        if i == 4 and nth else rng)
+                else:
+                    if t.startswith('-'):
+                        raise CroniterBadCronError(
+                            "[{0}] is not acceptable,\
+                            negative numbers not allowed".format(
+                                        expr_format))
+                    if not star_or_int_re.search(t):
+                        t = cls._alphaconv(i, t, expressions)
+
+                    try:
+                        t = int(t)
+                    except:
+                        pass
+
+                    if t in cls.LOWMAP[i]:
+                        t = cls.LOWMAP[i][t]
+
+                    if (
+                        t not in ["*", "l"]
+                        and (int(t) < cls.RANGES[i][0] or
+                             int(t) > cls.RANGES[i][1])
+                    ):
+                        raise CroniterBadCronError(
+                            "[{0}] is not acceptable, out of range".format(
+                                expr_format))
+
+                    res.append(t)
+
+                    if i == 4 and nth:
+                        if t not in nth_weekday_of_month:
+                            nth_weekday_of_month[t] = set()
+                        nth_weekday_of_month[t].add(int(nth))
+
+            res.sort()
+            expanded.append(['*'] if (len(res) == 1
+                                      and res[0] == '*')
+                            else res)
+
+        return expanded, nth_weekday_of_month
+
+    @classmethod
+    def is_valid(cls, expression):
+        try:
+            cls.expand(expression)
+        except CroniterError:
+            return False
+        else:
+            return True
