@@ -1,12 +1,11 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 import unittest
 from datetime import datetime
 from time import sleep
 import pytz
-from croniter import croniter
-
+from croniter import croniter, CroniterBadDateError, CroniterBadCronError, CroniterNotAlphaError
 from croniter.tests import base
 
 
@@ -146,6 +145,49 @@ class CroniterTest(base.TestCase):
         self.assertEqual(n3.day, 3)
         self.assertEqual(n3.year, 2010)
 
+    def testNthWeekDay(self):
+        base = datetime(2010, 2, 25)
+        itr = croniter('0 0 * * sat#1', base)
+        n1 = itr.get_next(datetime)
+        self.assertEqual(n1.isoweekday(), 6)
+        self.assertEqual(n1.day, 6)
+        self.assertEqual(n1.month, 3)
+        n2 = itr.get_next(datetime)
+        self.assertEqual(n2.isoweekday(), 6)
+        self.assertEqual(n2.day, 3)
+        self.assertEqual(n2.month, 4)
+
+        base = datetime(2010, 1, 25)
+        itr = croniter('0 0 * * wed#5', base)
+        n1 = itr.get_next(datetime)
+        self.assertEqual(n1.month, 3)
+        self.assertEqual(n1.day, 31)
+        self.assertEqual(n1.year, 2010)
+        n2 = itr.get_next(datetime)
+        self.assertEqual(n2.month, 6)
+        self.assertEqual(n2.day, 30)
+        self.assertEqual(n2.year, 2010)
+        n3 = itr.get_next(datetime)
+        self.assertEqual(n3.month, 9)
+        self.assertEqual(n3.day, 29)
+        self.assertEqual(n3.year, 2010)
+
+    def testWeekDayDayAnd(self):
+        base = datetime(2010, 1, 25)
+        itr = croniter('0 0 1 * mon', base, day_or=False)
+        n1 = itr.get_next(datetime)
+        self.assertEqual(n1.month, 2)
+        self.assertEqual(n1.day, 1)
+        self.assertEqual(n1.year, 2010)
+        n2 = itr.get_next(datetime)
+        self.assertEqual(n2.month, 3)
+        self.assertEqual(n2.day, 1)
+        self.assertEqual(n2.year, 2010)
+        n3 = itr.get_next(datetime)
+        self.assertEqual(n3.month, 11)
+        self.assertEqual(n3.day, 1)
+        self.assertEqual(n3.year, 2010)
+
     def testMonth(self):
         base = datetime(2010, 1, 25)
         itr = croniter('0 0 1 * *', base)
@@ -228,10 +270,12 @@ class CroniterTest(base.TestCase):
         self.assertRaises(TypeError, itr.get_next, str)
         self.assertRaises(ValueError, croniter, '* * * *')
         self.assertRaises(ValueError, croniter, '* * 5-1 * *')
-        self.assertRaises(KeyError, croniter, '* * * janu-jun *')
+        self.assertRaises(ValueError, croniter, '-90 * * * *')
+        self.assertRaises(ValueError, croniter, 'a * * * *')
+        self.assertRaises(ValueError, croniter, '* * * janu-jun *')
 
     def testSundayToThursdayWithAlphaConversion(self):
-        base = datetime(2010, 8, 25, 15, 56) #wednesday
+        base = datetime(2010, 8, 25, 15, 56)  # wednesday
         itr = croniter("30 22 * * sun-thu", base)
         next = itr.get_next(datetime)
 
@@ -306,6 +350,30 @@ class CroniterTest(base.TestCase):
         self.assertEqual(prev3.hour, 0)
         self.assertEqual(prev3.minute, 0)
 
+    def testPrevNthWeekDay(self):
+        base = datetime(2010, 8, 25, 15, 56)
+        itr = croniter('0 0 * * sat#1,sun#2', base)
+        prev1 = itr.get_prev(datetime)
+        self.assertEqual(prev1.year, base.year)
+        self.assertEqual(prev1.month, base.month)
+        self.assertEqual(prev1.day, 8)
+        self.assertEqual(prev1.hour, 0)
+        self.assertEqual(prev1.minute, 0)
+
+        prev2 = itr.get_prev(datetime)
+        self.assertEqual(prev2.year, base.year)
+        self.assertEqual(prev2.month, base.month)
+        self.assertEqual(prev2.day, 7)
+        self.assertEqual(prev2.hour, 0)
+        self.assertEqual(prev2.minute, 0)
+
+        prev3 = itr.get_prev(datetime)
+        self.assertEqual(prev3.year, base.year)
+        self.assertEqual(prev3.month, 7)
+        self.assertEqual(prev3.day, 11)
+        self.assertEqual(prev3.hour, 0)
+        self.assertEqual(prev3.minute, 0)
+
     def testPrevWeekDay2(self):
         base = datetime(2010, 8, 25, 15, 56)
         itr = croniter('10 0 * * 0', base)
@@ -388,6 +456,112 @@ class CroniterTest(base.TestCase):
         self.assertEqual(n6.month, 2)
         self.assertEqual(n6.day, 16)
 
+    def test_bug34(self):
+        base = datetime(2012, 2, 24, 0, 0, 0)
+        itr = croniter('* * 31 2 *', base)
+        try:
+            itr.get_next(datetime)
+        except (CroniterBadDateError,) as ex:
+            self.assertEqual("{0}".format(ex),
+                             'failed to find next date')
+
+    def testBug57(self):
+        base = datetime(2012, 2, 24, 0, 0, 0)
+        itr = croniter('0 4/6 * * *', base)
+        n1 = itr.get_next(datetime)
+        self.assertEqual(n1.hour, 4)
+        self.assertEqual(n1.minute, 0)
+        self.assertEqual(n1.month, 2)
+        self.assertEqual(n1.day, 24)
+
+        n1 = itr.get_prev(datetime)
+        self.assertEqual(n1.hour, 22)
+        self.assertEqual(n1.minute, 0)
+        self.assertEqual(n1.month, 2)
+        self.assertEqual(n1.day, 23)
+
+        itr = croniter('0 0/6 * * *', base)
+        n1 = itr.get_next(datetime)
+        self.assertEqual(n1.hour, 6)
+        self.assertEqual(n1.minute, 0)
+        self.assertEqual(n1.month, 2)
+        self.assertEqual(n1.day, 24)
+
+        n1 = itr.get_prev(datetime)
+        self.assertEqual(n1.hour, 0)
+        self.assertEqual(n1.minute, 0)
+        self.assertEqual(n1.month, 2)
+        self.assertEqual(n1.day, 24)
+
+    def test_multiple_months(self):
+        base = datetime(2016, 3, 1, 0, 0, 0)
+        itr = croniter('0 0 1 3,6,9,12 *', base)
+        n1 = itr.get_next(datetime)
+        self.assertEqual(n1.hour, 0)
+        self.assertEqual(n1.month, 6)
+        self.assertEqual(n1.day, 1)
+        self.assertEqual(n1.year, 2016)
+
+        base = datetime(2016, 2, 15, 0, 0, 0)
+        itr = croniter('0 0 1 3,6,9,12 *', base)
+        n1 = itr.get_next(datetime)
+        self.assertEqual(n1.hour, 0)
+        self.assertEqual(n1.month, 3)
+        self.assertEqual(n1.day, 1)
+        self.assertEqual(n1.year, 2016)
+
+        base = datetime(2016, 12, 3, 10, 0, 0)
+        itr = croniter('0 0 1 3,6,9,12 *', base)
+        n1 = itr.get_next(datetime)
+        self.assertEqual(n1.hour, 0)
+        self.assertEqual(n1.month, 3)
+        self.assertEqual(n1.day, 1)
+        self.assertEqual(n1.year, 2017)
+
+        # The result with this parameters was incorrect.
+        # self.assertEqual(p1.month, 12
+        # AssertionError: 9 != 12
+        base = datetime(2016, 3, 1, 0, 0, 0)
+        itr = croniter('0 0 1 3,6,9,12 *', base)
+        p1 = itr.get_prev(datetime)
+        self.assertEqual(p1.hour, 0)
+        self.assertEqual(p1.month, 12)
+        self.assertEqual(p1.day, 1)
+        self.assertEqual(p1.year, 2015)
+
+        # check my change resolves another hidden bug.
+        base = datetime(2016, 2, 1, 0, 0, 0)
+        itr = croniter('0 0 1,15,31 * *', base)
+        p1 = itr.get_prev(datetime)
+        self.assertEqual(p1.hour, 0)
+        self.assertEqual(p1.month, 1)
+        self.assertEqual(p1.day, 31)
+        self.assertEqual(p1.year, 2016)
+
+        base = datetime(2016, 6, 1, 0, 0, 0)
+        itr = croniter('0 0 1 3,6,9,12 *', base)
+        p1 = itr.get_prev(datetime)
+        self.assertEqual(p1.hour, 0)
+        self.assertEqual(p1.month, 3)
+        self.assertEqual(p1.day, 1)
+        self.assertEqual(p1.year, 2016)
+
+        base = datetime(2016, 3, 1, 0, 0, 0)
+        itr = croniter('0 0 1 1,3,6,9,12 *', base)
+        p1 = itr.get_prev(datetime)
+        self.assertEqual(p1.hour, 0)
+        self.assertEqual(p1.month, 1)
+        self.assertEqual(p1.day, 1)
+        self.assertEqual(p1.year, 2016)
+
+        base = datetime(2016, 3, 1, 0, 0, 0)
+        itr = croniter('0 0 1 1,3,6,9,12 *', base)
+        p1 = itr.get_prev(datetime)
+        self.assertEqual(p1.hour, 0)
+        self.assertEqual(p1.month, 1)
+        self.assertEqual(p1.day, 1)
+        self.assertEqual(p1.year, 2016)
+
     def test_rangeGenerator(self):
         base = datetime(2013, 3, 4, 0, 0)
         itr = croniter('1-9/2 0 1 * *', base)
@@ -467,13 +641,15 @@ class CroniterTest(base.TestCase):
         itr = croniter('* * * * *')
         sleep(.01)
         itr2 = croniter('* * * * *')
-        self.assertGreater(itr2.cur, itr.cur)
+        # Greater dosnt exists in py26
+        self.assertTrue(itr2.cur > itr.cur)
 
     def assertScheduleTimezone(self, callback, expected_schedule):
         for expected_date, expected_offset in expected_schedule:
             d = callback()
             self.assertEqual(expected_date, d.replace(tzinfo=None))
-            self.assertEqual(expected_offset, croniter._timedelta_to_seconds(d.utcoffset()))
+            self.assertEqual(expected_offset,
+                             croniter._timedelta_to_seconds(d.utcoffset()))
 
     def testTimezoneWinterTime(self):
         tz = pytz.timezone('Europe/Athens')
@@ -515,6 +691,147 @@ class CroniterTest(base.TestCase):
         ct = croniter('*/30 * * * *', tz.localize(start))
         self.assertScheduleTimezone(lambda: ct.get_prev(datetime), reversed(expected_schedule))
 
+    def test_std_dst(self):
+        """
+        DST tests
+
+        This fixes https://github.com/taichino/croniter/issues/82
+
+        """
+        tz = pytz.timezone('Europe/Warsaw')
+        # -> 2017-03-26 01:59+1:00 -> 03:00+2:00
+        local_date = tz.localize(datetime(2017, 3, 26))
+        val = croniter('0 0 * * *', local_date).get_next(datetime)
+        self.assertEqual(val, tz.localize(datetime(2017, 3, 27)))
+        #
+        local_date = tz.localize(datetime(2017, 3, 26, 1))
+        cr = croniter('0 * * * *', local_date)
+        val = cr.get_next(datetime)
+        self.assertEqual(val, tz.localize(datetime(2017, 3, 26, 3)))
+        val = cr.get_current(datetime)
+        self.assertEqual(val, tz.localize(datetime(2017, 3, 26, 3)))
+
+        # -> 2017-10-29 02:59+2:00 -> 02:00+1:00
+        local_date = tz.localize(datetime(2017, 10, 29))
+        val = croniter('0 0 * * *', local_date).get_next(datetime)
+        self.assertEqual(val, tz.localize(datetime(2017, 10, 30)))
+        local_date = tz.localize(datetime(2017, 10, 29, 1, 59))
+        val = croniter('0 * * * *', local_date).get_next(datetime)
+        self.assertEqual(
+            val.replace(tzinfo=None),
+            tz.localize(datetime(2017, 10, 29, 2)).replace(tzinfo=None))
+        local_date = tz.localize(datetime(2017, 10, 29, 2))
+        val = croniter('0 * * * *', local_date).get_next(datetime)
+        self.assertEqual(val, tz.localize(datetime(2017, 10, 29, 3)))
+        local_date = tz.localize(datetime(2017, 10, 29, 3))
+        val = croniter('0 * * * *', local_date).get_next(datetime)
+        self.assertEqual(val, tz.localize(datetime(2017, 10, 29, 4)))
+        local_date = tz.localize(datetime(2017, 10, 29, 4))
+        val = croniter('0 * * * *', local_date).get_next(datetime)
+        self.assertEqual(val, tz.localize(datetime(2017, 10, 29, 5)))
+        local_date = tz.localize(datetime(2017, 10, 29, 5))
+        val = croniter('0 * * * *', local_date).get_next(datetime)
+        self.assertEqual(val, tz.localize(datetime(2017, 10, 29, 6)))
+
+    def test_std_dst2(self):
+        """
+        DST tests
+
+        This fixes https://github.com/taichino/croniter/issues/87
+
+        São Paulo, Brazil: 18/02/2018 00:00 -> 17/02/2018 23:00
+
+        """
+        tz = pytz.timezone("America/Sao_Paulo")
+        local_dates = [
+            # 17-22: 00 -> 18-00:00
+            (tz.localize(datetime(2018, 2, 17, 21, 0, 0)),
+             '2018-02-18 00:00:00-03:00'),
+            # 17-23: 00 -> 18-00:00
+            (tz.localize(datetime(2018, 2, 17, 22, 0, 0)),
+             '2018-02-18 00:00:00-03:00'),
+            # 17-23: 00 -> 18-00:00
+            (tz.localize(datetime(2018, 2, 17, 23, 0, 0)),
+             '2018-02-18 00:00:00-03:00'),
+            # 18-00: 00 -> 19-00:00
+            (tz.localize(datetime(2018, 2, 18, 0, 0, 0)),
+             '2018-02-19 00:00:00-03:00'),
+            # 17-22: 00 -> 18-00:00
+            (tz.localize(datetime(2018, 2, 17, 21, 5, 0)),
+             '2018-02-18 00:00:00-03:00'),
+            # 17-23: 00 -> 18-00:00
+            (tz.localize(datetime(2018, 2, 17, 22, 5, 0)),
+             '2018-02-18 00:00:00-03:00'),
+            # 17-23: 00 -> 18-00:00
+            (tz.localize(datetime(2018, 2, 17, 23, 5, 0)),
+             '2018-02-18 00:00:00-03:00'),
+            # 18-00: 00 -> 19-00:00
+            (tz.localize(datetime(2018, 2, 18, 0, 5, 0)),
+             '2018-02-19 00:00:00-03:00'),
+        ]
+        ret1 = [croniter("0 0 * * *", d[0]).get_next(datetime)
+                for d in local_dates]
+        sret1 = ['{0}'.format(d) for d in ret1]
+        lret1 = ['{0}'.format(d[1]) for d in local_dates]
+        self.assertEqual(sret1, lret1)
+
+    def test_error_alpha_cron(self):
+        self.assertRaises(CroniterNotAlphaError, croniter.expand,
+                          '* * * janu-jun *')
+
+    def test_error_bad_cron(self):
+        self.assertRaises(CroniterBadCronError, croniter.expand,
+                          '* * * *')
+        self.assertRaises(CroniterBadCronError, croniter.expand,
+                          '* * * * * * *')
+
+    def test_is_valid(self):
+        self.assertTrue(croniter.is_valid('0 * * * *'))
+        self.assertFalse(croniter.is_valid('0 * *'))
+        self.assertFalse(croniter.is_valid('* * * janu-jun *'))
+
+    def test_exactly_the_same_minute(self):
+        base = datetime(2018, 3, 5, 12, 30, 50)
+        itr = croniter('30 7,12,17 * * *', base)
+        n1 = itr.get_prev(datetime)
+        self.assertEqual(12, n1.hour)
+
+        n2 = itr.get_prev(datetime)
+        self.assertEqual(7, n2.hour)
+
+        n3 = itr.get_next(datetime)
+        self.assertEqual(12, n3.hour)
+
+    def test_next_when_now_satisfies_cron(self):
+        ts_a = datetime(2018, 5, 21, 0, 3, 0)
+        ts_b = datetime(2018, 5, 21, 0, 4, 20)
+        test_cron = '4 * * * *'
+
+        next_a = croniter(test_cron, start_time=ts_a).get_next()
+        next_b = croniter(test_cron, start_time=ts_b).get_next()
+
+        self.assertTrue(next_b > next_a)
+
+    def test_milliseconds(self):
+        """
+        https://github.com/taichino/croniter/issues/107
+        """
+        #
+        dt = datetime(2018, 1, 2, 10, 0, 0, 500)
+        c = croniter("0 10 * * * ", start_time=dt)
+        ts = "{0}".format(datetime.utcfromtimestamp(c.get_prev()))
+        self.assertEqual(ts, '2018-01-02 10:00:00')
+        #
+        dt = datetime(2018, 1, 2, 10, 0, 1, 0)
+        c = croniter("0 10 * * * ", start_time=dt)
+        ts = "{0}".format(datetime.utcfromtimestamp(c.get_prev()))
+        self.assertEqual(ts, '2018-01-02 10:00:00')
+        #
+        dt = datetime(2018, 1, 2, 9, 59, 59, 999999)
+        c = croniter("0 10 * * * ", start_time=dt)
+        ts = "{0}".format(datetime.utcfromtimestamp(c.get_prev()))
+        self.assertEqual(ts, '2018-01-01 10:00:00')
+
 
 if __name__ == '__main__':
     unittest.main()
